@@ -2,9 +2,11 @@
 Base class for freezing scripts into executables.
 """
 
+from __future__ import print_function
+
 import datetime
 import distutils.sysconfig
-import imp
+from importlib.util import MAGIC_NUMBER
 import marshal
 import os
 import shutil
@@ -19,32 +21,6 @@ import cx_Freeze
 
 __all__ = [ "ConfigError", "ConstantsModule", "Executable", "Freezer" ]
 
-EXTENSION_LOADER_SOURCE = \
-"""
-def __bootstrap__():
-    import imp, os, sys
-    global __bootstrap__, __loader__
-    __loader__ = None; del __bootstrap__, __loader__
-
-    found = False
-    for p in sys.path:
-        if not os.path.isdir(p):
-            continue
-        f = os.path.join(p, "%s")
-        if not os.path.exists(f):
-            continue
-        m = imp.load_dynamic(__name__, f)
-        import sys
-        sys.modules[__name__] = m
-        found = True
-        break
-    if not found:
-        del sys.modules[__name__]
-        raise ImportError("No module named %%s" %% __name__)
-__bootstrap__()
-"""
-
-
 MSVCR_MANIFEST_TEMPLATE = """
 <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
@@ -63,11 +39,11 @@ MSVCR_MANIFEST_TEMPLATE = """
 
 def process_path_specs(specs):
     """Prepare paths specified as config.
-    
+
     The input is a list of either strings, or 2-tuples (source, target).
     Where single strings are supplied, the basenames are used as targets.
     Where targets are given explicitly, they must not be absolute paths.
-    
+
     Returns a list of 2-tuples, or throws ConfigError if something is wrong
     in the input.
     """
@@ -90,19 +66,33 @@ def process_path_specs(specs):
         processedSpecs.append((source, target))
     return processedSpecs
 
+def get_resource_file_path(dirName, name, ext):
+    """Return the path to a resource file shipped with cx_Freeze.
+
+    This is used to find our base executables and initscripts when they are
+    just specified by name.
+    """
+    if os.path.isabs(name):
+        return name
+    name = os.path.normcase(name)
+    fullDir = os.path.join(os.path.dirname(cx_Freeze.__file__), dirName)
+    if os.path.isdir(fullDir):
+        for fileName in os.listdir(fullDir):
+            checkName, checkExt = \
+                    os.path.splitext(os.path.normcase(fileName))
+            if name == checkName and ext == checkExt:
+                return os.path.join(fullDir, fileName)
+
 
 class Freezer(object):
 
     def __init__(self, executables, constantsModules = [], includes = [],
-            excludes = [], packages = [], replacePaths = [], compress = None,
-            optimizeFlag = 0, copyDependentFiles = None, initScript = None,
-            base = None, path = None, createLibraryZip = None,
-            appendScriptToExe = None, appendScriptToLibrary = None,
-            targetDir = None, binIncludes = [], binExcludes = [],
-            binPathIncludes = [], binPathExcludes = [], icon = None,
+            excludes = [], packages = [], replacePaths = [], compress = True,
+            optimizeFlag = 0, path = None, targetDir = None, binIncludes = [],
+            binExcludes = [], binPathIncludes = [], binPathExcludes = [],
             includeFiles = [], zipIncludes = [], silent = False,
-            namespacePackages = [], metadata = None,
-            includeMSVCR = False):
+            namespacePackages = [], metadata = None, includeMSVCR = False,
+            zipIncludePackages = [], zipExcludePackages = ["*"]):
         self.executables = list(executables)
         self.constantsModules = list(constantsModules)
         self.includes = list(includes)
@@ -112,14 +102,8 @@ class Freezer(object):
         self.replacePaths = list(replacePaths)
         self.compress = compress
         self.optimizeFlag = optimizeFlag
-        self.copyDependentFiles = copyDependentFiles
-        self.initScript = initScript
-        self.base = base
         self.path = path
-        self.createLibraryZip = createLibraryZip
         self.includeMSVCR = includeMSVCR
-        self.appendScriptToExe = appendScriptToExe
-        self.appendScriptToLibrary = appendScriptToLibrary
         self.targetDir = targetDir
         self.binIncludes = [os.path.normcase(n) \
                 for n in self._GetDefaultBinIncludes() + binIncludes]
@@ -128,28 +112,37 @@ class Freezer(object):
         self.binPathIncludes = [os.path.normcase(n) for n in binPathIncludes]
         self.binPathExcludes = [os.path.normcase(n) \
                 for n in self._GetDefaultBinPathExcludes() + binPathExcludes]
-        self.icon = icon
         self.includeFiles = process_path_specs(includeFiles)
         self.zipIncludes = process_path_specs(zipIncludes)
         self.silent = silent
         self.metadata = metadata
+        self.zipIncludePackages = list(zipIncludePackages)
+        self.zipExcludePackages = list(zipExcludePackages)
         self._VerifyConfiguration()
 
-    def _AddVersionResource(self, fileName):
+    def _AddVersionResource(self, exe):
+        warning_msg = "*** WARNING *** unable to create version resource"
         try:
             from win32verstamp import stamp
         except:
-            print("*** WARNING *** unable to create version resource")
+            print(warning_msg)
             print("install pywin32 extensions first")
             return
+        if not self.metadata.version:
+            print(warning_msg)
+            print("version must be specified")
+            return
+        fileName = exe.targetName
         versionInfo = VersionInfo(self.metadata.version,
                 comments = self.metadata.long_description,
                 description = self.metadata.description,
                 company = self.metadata.author,
-                product = self.metadata.name)
+                product = self.metadata.name,
+                copyright = exe.copyright,
+                trademarks = exe.trademarks)
         stamp(fileName, versionInfo)
 
-    def _CopyFile(self, source, target, copyDependentFiles = False,
+    def _CopyFile(self, source, target, copyDependentFiles,
             includeMode = False):
         normalizedSource = os.path.normcase(os.path.normpath(source))
         normalizedTarget = os.path.normcase(os.path.normpath(target))
@@ -167,7 +160,12 @@ class Freezer(object):
         if includeMode:
             shutil.copymode(source, target)
         self.filesCopied[normalizedTarget] = None
-        if copyDependentFiles:
+        if copyDependentFiles \
+                and source not in self.finder.excludeDependentFiles:
+            # Always copy dependent files on root directory
+            # to allow to set relative reference
+            if sys.platform == 'darwin':
+                targetDir = self.targetDir
             for source in self._GetDependentFiles(source):
                 target = os.path.join(targetDir, os.path.basename(source))
                 self._CopyFile(source, target, copyDependentFiles)
@@ -179,17 +177,19 @@ class Freezer(object):
             os.makedirs(path)
 
     def _FreezeExecutable(self, exe):
-        if self.createLibraryZip:
-            finder = self.finder
-        else:
-            finder = self._GetModuleFinder(exe)
-        if exe.script is None:
-            scriptModule = None
-        else:
-            scriptModule = finder.IncludeFile(exe.script, exe.moduleName)
+        finder = self.finder
+        finder.IncludeFile(exe.script, exe.moduleName)
+        finder.IncludeFile(exe.initScript, exe.initModuleName)
+        startupModule = get_resource_file_path("initscripts", "__startup__",
+                ".py")
+        finder.IncludeFile(startupModule)
 
-        self._CopyFile(exe.base, exe.targetName, exe.copyDependentFiles,
+        self._CopyFile(exe.base, exe.targetName, copyDependentFiles = True,
                 includeMode = True)
+        if not os.access(exe.targetName, os.W_OK):
+            mode = os.stat(exe.targetName).st_mode
+            os.chmod(exe.targetName, mode | stat.S_IWUSR)
+
         if self.includeMSVCR:
             self._IncludeMSVCR(exe)
 
@@ -204,39 +204,8 @@ class Freezer(object):
                 self._CopyFile(exe.icon, targetName,
                         copyDependentFiles = False)
 
-        if not os.access(exe.targetName, os.W_OK):
-            mode = os.stat(exe.targetName).st_mode
-            os.chmod(exe.targetName, mode | stat.S_IWUSR)
         if self.metadata is not None and sys.platform == "win32":
-            self._AddVersionResource(exe.targetName)
-
-        # Write the zip file of Python modules. If we're using a shared
-        # library.zip this is done by the Freeze method instead.
-        if not exe.appendScriptToLibrary:
-            if exe.appendScriptToExe:
-                fileName = exe.targetName
-            else:
-                baseFileName, ext = os.path.splitext(exe.targetName)
-                fileName = baseFileName + ".zip"
-                self._RemoveFile(fileName)
-            if not self.createLibraryZip and exe.copyDependentFiles:
-                scriptModule = None
-            self._WriteModules(fileName, exe.initScript, finder, exe.compress,
-                    exe.copyDependentFiles, scriptModule)
-
-    def _GetBaseFileName(self, argsSource = None):
-        if argsSource is None:
-            argsSource = self
-        name = argsSource.base
-        if name is None:
-            if argsSource.copyDependentFiles:
-                name = "Console"
-            else:
-                name = "ConsoleKeepPath"
-        ext = ".exe" if sys.platform == "win32" else ""
-        argsSource.base = self._GetFileName("bases", name, ext)
-        if argsSource.base is None:
-            raise ConfigError("no base named %s", name)
+            self._AddVersionResource(exe)
 
     def _GetDefaultBinExcludes(self):
         """Return the file names of libraries that need not be included because
@@ -281,21 +250,29 @@ class Freezer(object):
         """Return the file's dependencies using platform-specific tools (the
            imagehlp library on Windows, otool on Mac OS X and ldd on Linux);
            limit this list by the exclusion lists as needed"""
+        dirname = os.path.dirname(path)
         dependentFiles = self.dependentFiles.get(path)
         if dependentFiles is None:
             if sys.platform == "win32":
-                origPath = os.environ["PATH"]
-                os.environ["PATH"] = origPath + os.pathsep + \
-                        os.pathsep.join(sys.path)
-                import cx_Freeze.util
-                try:
-                    dependentFiles = cx_Freeze.util.GetDependentFiles(path)
-                except cx_Freeze.util.BindError:
-                    # Sometimes this gets called when path is not actually a library
-                    # See issue 88
+                if path.endswith(('.exe', '.dll')):
+                    origPath = os.environ["PATH"]
+                    os.environ["PATH"] = origPath + os.pathsep + \
+                            os.pathsep.join(sys.path)
+                    try:
+                        dependentFiles = cx_Freeze.util.GetDependentFiles(path)
+                    except cx_Freeze.util.BindError as exc:
+                        # Sometimes this gets called when path is not actually a
+                        # library See issue 88
+                        dependentFiles = []
+                        fmt = "error during GetDependentFiles() of \"%s\": %s\n"
+                        sys.stderr.write(fmt % (path, str(exc)))
+                    os.environ["PATH"] = origPath
+                else:
                     dependentFiles = []
-                os.environ["PATH"] = origPath
             else:
+                if not os.access(path, os.X_OK):
+                    self.dependentFiles[path] = []
+                    return []
                 dependentFiles = []
                 if sys.platform == "darwin":
                     command = 'otool -L "%s"' % path
@@ -310,6 +287,8 @@ class Freezer(object):
                     if len(parts) != 2:
                         continue
                     dependentFile = parts[dependentFileIndex].strip()
+                    if dependentFile == os.path.basename(path):
+                        continue
                     if dependentFile in ("not found", "(file not found)"):
                         fileName = parts[0]
                         if fileName not in self.linkerWarnings:
@@ -329,52 +308,40 @@ class Freezer(object):
                     # cx_Freeze on OSX in e.g. a conda-based distribution.
                     # Note that with @rpath we just assume Python's lib dir,
                     # which should work in most cases.
-                    dirname = os.path.dirname(path)
                     dependentFiles = [p.replace('@loader_path', dirname)
                                       for p in dependentFiles]
                     dependentFiles = [p.replace('@rpath', sys.prefix + '/lib')
                                       for p in dependentFiles]
             dependentFiles = self.dependentFiles[path] = \
-                    [f for f in dependentFiles if self._ShouldCopyFile(f)]
+                    [self._CheckDependentFile(f, dirname) \
+                            for f in dependentFiles if self._ShouldCopyFile(f)]
         return dependentFiles
 
-    def _GetFileName(self, dirName, name, ext):
-        if os.path.isabs(name):
-            return name
-        name = os.path.normcase(name)
-        fullDir = os.path.join(os.path.dirname(cx_Freeze.__file__), dirName)
-        if os.path.isdir(fullDir):
-            for fileName in os.listdir(fullDir):
-                checkName, checkExt = \
-                        os.path.splitext(os.path.normcase(fileName))
-                if name == checkName and ext == checkExt:
-                    return os.path.join(fullDir, fileName)
-
-    def _GetInitScriptFileName(self, argsSource = None):
-        if argsSource is None:
-            argsSource = self
-        name = argsSource.initScript
-        if name is None:
-            if argsSource.copyDependentFiles:
-                name = "Console"
-            else:
-                name = "ConsoleKeepPath"
-        argsSource.initScript = self._GetFileName("initscripts", name, ".py")
-        if argsSource.initScript is None:
-            raise ConfigError("no initscript named %s", name)
+    def _CheckDependentFile(self, dependentFile, dirname):
+        """If the file does not exist, try to locate it in the directory of the
+           parent file (this is to workaround an issue in how otool returns
+           dependencies. See issue #292.
+           https://github.com/anthony-tuininga/cx_Freeze/issues/292"""
+        if os.path.isfile(dependentFile):
+            return dependentFile
+        basename = os.path.basename(dependentFile)
+        joined = os.path.join(dirname, basename)
+        if os.path.isfile(joined):
+            return joined
+        raise FileNotFoundError("otool returned a dependent file that "
+                "could not be found: " + dependentFile)
 
     def _GetModuleFinder(self, argsSource = None):
         if argsSource is None:
             argsSource = self
-        finder = cx_Freeze.ModuleFinder(self.includeFiles, argsSource.excludes,
-                argsSource.path, argsSource.replacePaths,
-                argsSource.copyDependentFiles, compress = argsSource.compress)
-        for name in argsSource.namespacePackages:
+        finder = cx_Freeze.ModuleFinder(self.includeFiles, self.excludes,
+                self.path, self.replacePaths)
+        for name in self.namespacePackages:
             package = finder.IncludeModule(name, namespace = True)
             package.ExtendPath()
-        for name in argsSource.includes:
+        for name in self.includes:
             finder.IncludeModule(name)
-        for name in argsSource.packages:
+        for name in self.packages:
             finder.IncludePackage(name)
         return finder
 
@@ -390,7 +357,8 @@ class Freezer(object):
                     if not os.path.exists(sourceName):
                         continue
                     targetName = os.path.join(targetDir, otherName)
-                    self._CopyFile(sourceName, targetName)
+                    self._CopyFile(sourceName, targetName,
+                            copyDependentFiles = False)
                 break
 
         if msvcRuntimeDll is not None and msvcRuntimeDll == "msvcr90.dll":
@@ -416,8 +384,6 @@ class Freezer(object):
             sys.stdout.write(" %-25s %s\n" % (module.name, module.file or ""))
         sys.stdout.write("\n")
 
-    
-
     def _RemoveFile(self, path):
         if os.path.exists(path):
             os.chmod(path, stat.S_IWRITE)
@@ -441,7 +407,7 @@ class Freezer(object):
            binIncludes and binExcludes configuration variables using first the
            full file name, then just the base file name, then the file name
            without any version numbers.
-           
+
            Files are included unless specifically excluded but inclusions take
            precedence over exclusions."""
 
@@ -481,31 +447,24 @@ class Freezer(object):
 
         return True
 
-    def _VerifyCanAppendToLibrary(self):
-        if not self.createLibraryZip:
-            raise ConfigError("script cannot be appended to library zip if "
-                    "one is not being created")
+    def _ShouldIncludeInFileSystem(self, module):
+        if module.parent is not None:
+            return self._ShouldIncludeInFileSystem(module.parent)
+        if module.path is None or module.file is None:
+            return False
+        if self.zipIncludeAllPackages \
+                and module.name not in self.zipExcludePackages \
+                or module.name in self.zipIncludePackages:
+            return False
+        return True
 
     def _VerifyConfiguration(self):
         if self.compress is None:
             self.compress = True
-        if self.copyDependentFiles is None:
-            self.copyDependentFiles = True
-        if self.createLibraryZip is None:
-            self.createLibraryZip = True
-        if self.appendScriptToExe is None:
-            self.appendScriptToExe = False
-        if self.appendScriptToLibrary is None:
-            self.appendScriptToLibrary = \
-                    self.createLibraryZip and not self.appendScriptToExe
         if self.targetDir is None:
             self.targetDir = os.path.abspath("dist")
-        self._GetInitScriptFileName()
-        self._GetBaseFileName()
         if self.path is None:
             self.path = sys.path
-        if self.appendScriptToLibrary:
-            self._VerifyCanAppendToLibrary()
 
         for sourceFileName, targetFileName in \
                 self.includeFiles + self.zipIncludes:
@@ -515,80 +474,118 @@ class Freezer(object):
             if os.path.isabs(targetFileName):
                 raise ConfigError("target file/directory cannot be absolute")
 
+        self.zipExcludeAllPackages = "*" in self.zipExcludePackages
+        self.zipIncludeAllPackages = "*" in self.zipIncludePackages
+        if self.zipExcludeAllPackages and self.zipIncludeAllPackages:
+            raise ConfigError("all packages cannot be included and excluded " \
+                    "from the zip file at the same time")
+        for name in self.zipIncludePackages:
+            if name in self.zipExcludePackages:
+                raise ConfigError("package %s cannot be both included and " \
+                        "excluded from zip file", name)
+
         for executable in self.executables:
             executable._VerifyConfiguration(self)
 
-    def _WriteModules(self, fileName, initScript, finder, compress,
-            copyDependentFiles, scriptModule = None):
-        initModule = finder.IncludeFile(initScript, "cx_Freeze__init__")
-        if scriptModule is None:
-            for module in self.constantsModules:
-                module.Create(finder)
-            modules = [m for m in finder.modules \
-                    if m.name not in self.excludeModules]
-        else:
-            modules = [initModule, scriptModule]
-            self.excludeModules[initModule.name] = None
-            self.excludeModules[scriptModule.name] = None
+    def _WriteModules(self, fileName, finder):
+        for module in self.constantsModules:
+            module.Create(finder)
+        modules = [m for m in finder.modules \
+                if m.name not in self.excludeModules]
         modules.sort(key = lambda m: m.name)
+
         if not self.silent:
             self._PrintReport(fileName, modules)
-        if scriptModule is None:
-            finder.ReportMissingModules()
+        finder.ReportMissingModules()
 
         targetDir = os.path.dirname(fileName)
         self._CreateDirectory(targetDir)
 
-        # Prepare zip file. This can be library.zip, or named after the
-        # executable, or even appended to the executable.
-        if os.path.exists(fileName):
-            mode = "a"
-        else:
-            mode = "w"
-        outFile = zipfile.PyZipFile(fileName, mode, zipfile.ZIP_DEFLATED)
+        # Prepare zip file
+        outFile = zipfile.PyZipFile(fileName, "w", zipfile.ZIP_DEFLATED)
 
         filesToCopy = []
+        ignorePatterns = shutil.ignore_patterns("*.py", "*.pyc", "*.pyo",
+                "__pycache__")
         for module in modules:
-            if module.code is None and module.file is not None:
-                # Extension module: save a Python loader in the zip file, and
-                # copy the actual file to the build directory, because pyd/so
-                # libraries can't be loaded from a zip file.
-                fileName = os.path.basename(module.file)
-                baseFileName, ext = os.path.splitext(fileName)
-                if baseFileName != module.name and module.name != "zlib":
-                    if "." in module.name:
-                        fileName = module.name + ext
-                    generatedFileName = "ExtensionLoader_%s.py" % \
-                            module.name.replace(".", "_")
-                    module.code = compile(EXTENSION_LOADER_SOURCE % fileName,
-                            generatedFileName, "exec")
-                target = os.path.join(targetDir, fileName)
+
+            # determine if the module should be written to the file system;
+            # a number of packages make the assumption that files that they
+            # require will be found in a location relative to where
+            # they are located on disk; these packages will fail with strange
+            # errors when they are written to a zip file instead
+            includeInFileSystem = self._ShouldIncludeInFileSystem(module)
+
+            # if the module refers to a package, check to see if this package
+            # should be included in the zip file or should be written to the
+            # file system; if the package should be written to the file system,
+            # any non-Python files are copied at this point if the target
+            # directory does not already exist
+            if module.path is not None and includeInFileSystem:
+                parts = module.name.split(".")
+                targetPackageDir = os.path.join(targetDir, *parts)
+                sourcePackageDir = os.path.dirname(module.file)
+                if not os.path.exists(targetPackageDir):
+                    print("Copying data from package", module.name + "...")
+                    shutil.copytree(sourcePackageDir, targetPackageDir,
+                            ignore = ignorePatterns)
+
+            # if an extension module is found in a package that is to be
+            # included in a zip file, save a Python loader in the zip file and
+            # copy the actual file to the build directory because shared
+            # libraries cannot be loaded from a zip file
+            if module.code is None and module.file is not None \
+                    and not includeInFileSystem:
+                parts = module.name.split(".")[:-1]
+                parts.append(os.path.basename(module.file))
+                target = os.path.join(targetDir, ".".join(parts))
                 filesToCopy.append((module, target))
 
-            if module.code is None:
-                continue
-
-            fileName = "/".join(module.name.split("."))
-            if module.path:
-                fileName += "/__init__"
-            if module.file is not None and os.path.exists(module.file):
-                mtime = os.stat(module.file).st_mtime
-            else:
-                mtime = time.time()
-            zipTime = time.localtime(mtime)[:6]
             # starting with Python 3.3 the pyc file format contains the source
             # size; it is not actually used for anything except determining if
             # the file is up to date so we can safely set this value to zero
-            if sys.version_info[:2] < (3, 3):
-                header = imp.get_magic() + struct.pack("<i", int(mtime))
-            else:
-                header = imp.get_magic() + struct.pack("<ii", int(mtime), 0)
-            data = header + marshal.dumps(module.code)
-            zinfo = zipfile.ZipInfo(fileName + ".pyc", zipTime)
-            if compress:
-                zinfo.compress_type = zipfile.ZIP_DEFLATED
-            outFile.writestr(zinfo, data)
+            if module.code is not None:
+                if module.file is not None and os.path.exists(module.file):
+                    stat = os.stat(module.file)
+                    mtime = stat.st_mtime
+                    size = stat.st_size & 0xFFFFFFFF
+                else:
+                    mtime = time.time()
+                    size = 0
+                if sys.version_info[:2] < (3, 7):
+                    header = MAGIC_NUMBER + struct.pack("<ii", int(mtime), size)
+                else:
+                    header = MAGIC_NUMBER + struct.pack("<iii", 0, int(mtime), size)
+                data = header + marshal.dumps(module.code)
 
+            # if the module should be written to the file system, do so
+            if includeInFileSystem:
+                parts = module.name.split(".")
+                if module.code is None:
+                    parts.pop()
+                    parts.append(os.path.basename(module.file))
+                    targetName = os.path.join(targetDir, *parts)
+                    self._CopyFile(module.file, targetName,
+                            copyDependentFiles = True)
+                else:
+                    if module.path is not None:
+                        parts.append("__init__")
+                    targetName = os.path.join(targetDir, *parts) + ".pyc"
+                    open(targetName, "wb").write(data)
+
+
+            # otherwise, write to the zip file
+            elif module.code is not None:
+                zipTime = time.localtime(mtime)[:6]
+                fileName = "/".join(module.name.split("."))
+                if module.path:
+                    fileName += "/__init__"
+                zinfo = zipfile.ZipInfo(fileName + ".pyc", zipTime)
+                if self.compress:
+                    zinfo.compress_type = zipfile.ZIP_DEFLATED
+                outFile.writestr(zinfo, data)
+
+        # write any files to the zip file that were requested specially
         for sourceFileName, targetFileName in self.zipIncludes:
             outFile.write(sourceFileName, targetFileName)
 
@@ -601,7 +598,7 @@ class Freezer(object):
                 if module.parent is not None:
                     path = os.pathsep.join([origPath] + module.parent.path)
                     os.environ["PATH"] = path
-                self._CopyFile(module.file, target, copyDependentFiles)
+                self._CopyFile(module.file, target, copyDependentFiles = True)
             finally:
                 os.environ["PATH"] = origPath
 
@@ -615,39 +612,38 @@ class Freezer(object):
         import cx_Freeze.util
         cx_Freeze.util.SetOptimizeFlag(self.optimizeFlag)
 
-        if self.createLibraryZip:
-            self.finder = self._GetModuleFinder()
+        self.finder = self._GetModuleFinder()
         for executable in self.executables:
             self._FreezeExecutable(executable)
-        if self.createLibraryZip:
-            fileName = os.path.join(self.targetDir, "library.zip")
-            self._RemoveFile(fileName)
-            self._WriteModules(fileName, self.initScript, self.finder,
-                    self.compress, self.copyDependentFiles)
+        targetDir = self.targetDir
+        zipTargetDir = os.path.join(targetDir, "lib")
+        fileName = os.path.join(zipTargetDir, "library.zip")
+        self._RemoveFile(fileName)
+        self._WriteModules(fileName, self.finder)
 
         for sourceFileName, targetFileName in self.includeFiles:
             if os.path.isdir(sourceFileName):
                 # Copy directories by recursing into them.
-                # TODO: Can we use shutil.copytree here?
+                # Can't use shutil.copytree because we may need dependencies
                 for path, dirNames, fileNames in os.walk(sourceFileName):
                     shortPath = path[len(sourceFileName) + 1:]
                     if ".svn" in dirNames:
                         dirNames.remove(".svn")
                     if "CVS" in dirNames:
                         dirNames.remove("CVS")
-                    fullTargetDir = os.path.join(self.targetDir,
+                    fullTargetDir = os.path.join(targetDir,
                             targetFileName, shortPath)
                     self._CreateDirectory(fullTargetDir)
                     for fileName in fileNames:
                         fullSourceName = os.path.join(path, fileName)
                         fullTargetName = os.path.join(fullTargetDir, fileName)
                         self._CopyFile(fullSourceName, fullTargetName,
-                                copyDependentFiles = False)
+                                copyDependentFiles = True)
             else:
                 # Copy regular files.
-                fullName = os.path.join(self.targetDir, targetFileName)
+                fullName = os.path.join(targetDir, targetFileName)
                 self._CopyFile(sourceFileName, fullName,
-                        copyDependentFiles = False)
+                        copyDependentFiles = True)
 
 
 class ConfigError(Exception):
@@ -661,80 +657,48 @@ class ConfigError(Exception):
 
 class Executable(object):
 
-    def __init__(self, script, initScript = None, base = None, path = None,
-            targetDir = None, targetName = None, includes = None,
-            excludes = None, packages = None, replacePaths = None,
-            compress = None, copyDependentFiles = None,
-            appendScriptToExe = None, appendScriptToLibrary = None,
-            icon = None, namespacePackages = None, shortcutName = None,
-            shortcutDir = None):
+    def __init__(self, script, initScript = None, base = None,
+            targetName = None, icon = None, shortcutName = None,
+            shortcutDir = None, copyright = None, trademarks = None):
         self.script = script
-        self.initScript = initScript
-        self.base = base
-        self.path = path
-        self.targetDir = targetDir
+        self.initScript = initScript or "Console"
+        self.base = base or "Console"
         self.targetName = targetName
-        self.includes = includes
-        self.excludes = excludes
-        self.packages = packages
-        self.namespacePackages = namespacePackages
-        self.replacePaths = replacePaths
-        self.compress = compress
-        self.copyDependentFiles = copyDependentFiles
-        self.appendScriptToExe = appendScriptToExe
-        self.appendScriptToLibrary = appendScriptToLibrary
         self.icon = icon
         self.shortcutName = shortcutName
         self.shortcutDir = shortcutDir
+        self.copyright = copyright
+        self.trademarks = trademarks
 
     def __repr__(self):
         return "<Executable script=%s>" % self.script
 
     def _VerifyConfiguration(self, freezer):
-        if self.path is None:
-            self.path = freezer.path
-        if self.targetDir is None:
-            self.targetDir = freezer.targetDir
-        if self.includes is None:
-            self.includes = freezer.includes
-        if self.excludes is None:
-            self.excludes = freezer.excludes
-        if self.packages is None:
-            self.packages = freezer.packages
-        if self.namespacePackages is None:
-            self.namespacePackages = freezer.namespacePackages
-        if self.replacePaths is None:
-            self.replacePaths = freezer.replacePaths
-        if self.compress is None:
-            self.compress = freezer.compress
-        if self.copyDependentFiles is None:
-            self.copyDependentFiles = freezer.copyDependentFiles
-        if self.appendScriptToExe is None:
-            self.appendScriptToExe = freezer.appendScriptToExe
-        if self.appendScriptToLibrary is None:
-            self.appendScriptToLibrary = freezer.appendScriptToLibrary
-        if self.initScript is None:
-            self.initScript = freezer.initScript
-        else:
-            freezer._GetInitScriptFileName(self)
-        if self.base is None:
-            self.base = freezer.base
-        else:
-            freezer._GetBaseFileName(self)
-        if self.appendScriptToLibrary:
-            freezer._VerifyCanAppendToLibrary()
-        if self.icon is None:
-            self.icon = freezer.icon
+        self._GetInitScriptFileName()
+        self._GetBaseFileName()
         if self.targetName is None:
-            name, ext = os.path.splitext(os.path.basename(self.script))
+            name, _ = os.path.splitext(os.path.basename(self.script))
             baseName, ext = os.path.splitext(self.base)
             self.targetName = name + ext
-        if self.appendScriptToLibrary:
-            name, ext = os.path.splitext(self.targetName)
-            self.moduleName = "%s__main__" % os.path.normcase(name)
-        else:
-            self.moduleName = "__main__"
-        self.targetName = os.path.join(self.targetDir, self.targetName)
+        name, ext = os.path.splitext(self.targetName)
+        if ext == "" and sys.platform == "win32":
+            self.targetName += ".exe"
+        self.moduleName = "%s__main__" % os.path.normcase(name)
+        self.initModuleName = "%s__init__" % os.path.normcase(name)
+        self.targetName = os.path.join(freezer.targetDir, self.targetName)
+
+    def _GetBaseFileName(self):
+        name = self.base
+        ext = ".exe" if sys.platform == "win32" else ""
+        self.base = get_resource_file_path("bases", name, ext)
+        if self.base is None:
+            raise ConfigError("no base named %s", name)
+
+    def _GetInitScriptFileName(self):
+        name = self.initScript
+        self.initScript = get_resource_file_path("initscripts", name, ".py")
+        if self.initScript is None:
+            raise ConfigError("no initscript named %s", name)
 
 
 class ConstantsModule(object):
