@@ -1,3 +1,4 @@
+import glob
 import os
 import sys
 
@@ -6,6 +7,7 @@ def initialize(finder):
        automatic exclusions for various platforms."""
     finder.ExcludeModule("FCNTL")
     finder.ExcludeModule("os.path")
+    finder.ExcludeModule("multiprocessing.Process")
     if os.name == "nt":
         finder.ExcludeModule("fcntl")
         finder.ExcludeModule("grp")
@@ -67,11 +69,13 @@ def initialize(finder):
         finder.ExcludeModule("org.python.core")
     if sys.platform[:4] != "OpenVMS":
         finder.ExcludeModule("vms_lib")
-    if sys.version_info[0] >= 3:
-        finder.ExcludeModule("new")
-        finder.ExcludeModule("Tkinter")
-    else:
-        finder.ExcludeModule("tkinter")
+    finder.ExcludeModule("new")
+    finder.ExcludeModule("Tkinter")
+
+
+def load_asyncio(finder, module):
+    """the asyncio must be loaded as a package."""
+    finder.IncludePackage('asyncio')
 
 
 def load_cElementTree(finder, module):
@@ -117,11 +121,10 @@ def load_dummy_threading(finder, module):
 def load_email(finder, module):
     """the email package has a bunch of aliases as the submodule names were
        all changed to lowercase in Python 2.5; mimic that here."""
-    if sys.version_info[:2] >= (2, 5):
-        for name in ("Charset", "Encoders", "Errors", "FeedParser",
-                "Generator", "Header", "Iterators", "Message", "Parser",
-                "Utils", "base64MIME", "quopriMIME"):
-            finder.AddAlias("email.%s" % name, "email.%s" % name.lower())
+    for name in ("Charset", "Encoders", "Errors", "FeedParser",
+            "Generator", "Header", "Iterators", "Message", "Parser",
+            "Utils", "base64MIME", "quopriMIME"):
+        finder.AddAlias("email.%s" % name, "email.%s" % name.lower())
 
 
 def load_ftplib(finder, module):
@@ -130,6 +133,11 @@ def load_ftplib(finder, module):
     module.IgnoreName("SOCKS")
 
 
+def load_gevent(finder, module):
+    """gevent must be loaded as a package."""
+    finder.IncludePackage("gevent")
+
+
 def load_GifImagePlugin(finder, module):
     """The GifImagePlugin module optionally imports the _imaging_gif module"""
     module.IgnoreName("_imaging_gif")
@@ -234,16 +242,30 @@ def load_h5py(finder, module):
     finder.IncludeModule('h5py.defs')
     finder.IncludeModule('h5py.utils')
     finder.IncludeModule('h5py._proxy')
-    finder.IncludeModule('h5py.api_gen')
+    try:
+        import h5py.api_gen
+        finder.IncludeModule('h5py.api_gen')
+    except ImportError:
+        pass
+    finder.IncludeModule('h5py._errors')
+    finder.IncludeModule('h5py.h5ac')
 
 
+def load_idna(finder, module):
+    """the idna module implicitly loads data; make sure this happens."""
+    finder.IncludeModule("idna.idnadata")
+
 def load_matplotlib(finder, module):
     """the matplotlib module requires data to be found in mpl-data in the
        same directory as the frozen executable so oblige it"""
-    dir = os.path.join(module.path[0], "mpl-data")
-    finder.IncludeFiles(dir, "mpl-data")
+    import matplotlib
+    dataPath = matplotlib.get_data_path()
+    finder.IncludeFiles(dataPath, "mpl-data", copyDependentFiles=False)
 
 
+def load_numpy(finder, module):
+    finder.IncludePackage("numpy")
+
 def load_matplotlib_numerix(finder, module):
     """the numpy.numerix module loads a number of modules dynamically"""
     for name in ("ma", "fft", "linear_algebra", "random_array", "mlab"):
@@ -375,6 +397,16 @@ def load_numpy_random_mtrand(finder, module):
     module.AddGlobalName("randn")
 
 
+def load_PIL(finder, module):
+    """Pillow must be loaded as a package."""
+    finder.IncludePackage("PIL")
+
+
+def load_pkg_resources(finder, module):
+    """pkg_resources dynamic load modules in a subpackage."""
+    finder.IncludePackage("pkg_resources._vendor")
+
+
 def load_postgresql_lib(finder, module):
     """the postgresql.lib module requires the libsys.sql file to be included
        so make sure that file is included"""
@@ -386,6 +418,12 @@ def load_pty(finder, module):
     """The sgi module is not needed for this module to function."""
     module.IgnoreName("sgi")
 
+def load_pycparser(finder, module):
+    """ These files are missing which causes
+        permission denied issues on windows when they are regenerated.
+    """
+    finder.IncludeModule("pycparser.lextab")
+    finder.IncludeModule("pycparser.yacctab")
 
 def load_pydoc(finder, module):
     """The pydoc module will work without the Tkinter module so ignore the
@@ -397,24 +435,25 @@ def load_pythoncom(finder, module):
     """the pythoncom module is actually contained in a DLL but since those
        cannot be loaded directly in Python 2.5 and higher a special module is
        used to perform that task; simply use that technique directly to
-       determine the name of the DLL and ensure it is included as a normal
-       extension; also load the pywintypes module which is implicitly
-       loaded."""
+       determine the name of the DLL and ensure it is included as a file in
+       the target directory."""
     import pythoncom
-    module.file = pythoncom.__file__
-    module.code = None
-    finder.IncludeModule("pywintypes")
+    finder.IncludeFiles(pythoncom.__file__,
+            os.path.join("lib", os.path.basename(pythoncom.__file__)),
+            copyDependentFiles = False)
 
 
 def load_pywintypes(finder, module):
     """the pywintypes module is actually contained in a DLL but since those
        cannot be loaded directly in Python 2.5 and higher a special module is
        used to perform that task; simply use that technique directly to
-       determine the name of the DLL and ensure it is included as a normal
-       extension."""
+       determine the name of the DLL and ensure it is included as a file in the
+       target directory."""
     import pywintypes
-    module.file = pywintypes.__file__
-    module.code = None
+    finder.IncludeFiles(pywintypes.__file__,
+            os.path.join("lib", os.path.basename(pywintypes.__file__)),
+            copyDependentFiles = False)
+
 
 # PyQt5 and PyQt4 can't both be loaded in the same process, so we cache the
 # QtCore module so we can still return something sensible if we try to load
@@ -434,7 +473,7 @@ def _qt_implementation(module):
 
 def copy_qt_plugins(plugins, finder, QtCore):
     """Helper function to find and copy Qt plugins."""
-    
+
     # Qt Plugins can either be in a plugins directory next to the Qt libraries,
     # or in other locations listed by QCoreApplication.libraryPaths()
     dir0 = os.path.join(os.path.dirname(QtCore.__file__), "plugins")
@@ -495,7 +534,10 @@ def load_PyQt4_uic(finder, module):
     dir = os.path.join(module.path[0], "widget-plugins")
     finder.IncludeFiles(dir, "%s.uic.widget-plugins" % name)
     finder.IncludeModule("%s.QtNetwork" % name)
-    finder.IncludeModule("%s.QtWebKit" % name)
+    try:
+        finder.IncludeModule("%s.QtWebKit" % name)
+    except ImportError:
+        pass
 
 load_PyQt5_uic = load_PyQt4_uic
 
@@ -535,6 +577,16 @@ def load_PyQt4_QtWebKit(finder, module):
 
 load_PyQt5_QtWebKit = load_PySide_QtWebKit = load_PyQt4_QtWebKit
 
+def load_PyQt5_QtMultimedia(finder, module):
+    name, QtCore = _qt_implementation(module)
+    finder.IncludeModule("%s.QtCore" % name)
+    finder.IncludeModule("%s.QtMultimediaWidgets" % name)
+    copy_qt_plugins("mediaservice", finder, QtCore)
+
+def load_PyQt5_QtPrintSupport(finder, module):
+    name, QtCore = _qt_implementation(module)
+    copy_qt_plugins("printsupport", finder, QtCore)
+
 def load_reportlab(finder, module):
     """the reportlab module loads a submodule rl_settings via exec so force
        its inclusion here"""
@@ -545,7 +597,7 @@ def load_scipy(finder, module):
     """the scipy module loads items within itself in a way that causes
        problems without the entire package and a number of other subpackages
        being present."""
-    finder.IncludePackage("scipy.lib")
+    finder.IncludePackage("scipy._lib")
     finder.IncludePackage("scipy.misc")
 
 
@@ -587,6 +639,16 @@ def load_site(finder, module):
     module.IgnoreName("usercustomize")
 
 
+def load_ssl(finder, module):
+    """In Windows, the SSL module in Python >= 3.7 requires additional dlls to
+       be present in the build directory."""
+    if sys.platform == "win32" and sys.version_info >= (3, 7):
+        for dll_search in ["libcrypto-*.dll", "libssl-*.dll"]:
+            for dll_path in glob.glob(os.path.join(sys.base_prefix, "DLLs", dll_search)):
+                dll_name = os.path.basename(dll_path)
+                finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+
+
 def load_tkinter(finder, module):
     """the tkinter module has data files that are required to be loaded so
        ensure that they are copied into the directory that is expected at
@@ -594,10 +656,21 @@ def load_tkinter(finder, module):
     if sys.platform == "win32":
         import tkinter
         import _tkinter
-        tclSourceDir = os.environ["TCL_LIBRARY"]
-        tkSourceDir = os.environ["TK_LIBRARY"]
-        finder.IncludeFiles(tclSourceDir, "tcl")
-        finder.IncludeFiles(tkSourceDir, "tk")
+        root_names = "tcl", "tk"
+        environ_names = "TCL_LIBRARY", "TK_LIBRARY"
+        version_vars = tkinter.TclVersion, tkinter.TkVersion
+        zipped = zip(environ_names, version_vars, root_names)
+        for env_name, ver_var, mod_name in zipped:
+            try:
+                lib_texts = os.environ[env_name]
+            except KeyError:
+                lib_texts = os.path.join(sys.base_prefix, "tcl",
+                        mod_name + str(ver_var))
+            finder.IncludeFiles(lib_texts, mod_name)
+        for ver_var, mod_name in zip(version_vars, root_names):
+            dll_name = mod_name + str(ver_var).replace(".", "") + "t.dll"
+            dll_path = os.path.join(sys.base_prefix, "Dlls", dll_name)
+            finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
 
 
 def load_Tkinter(finder, module):
@@ -650,6 +723,7 @@ def load_twitter(finder, module):
 def load_win32api(finder, module):
     """the win32api module implicitly loads the pywintypes module; make sure
        this happens."""
+    finder.ExcludeDependentFiles(module.file)
     finder.IncludeModule("pywintypes")
 
 
@@ -662,9 +736,10 @@ def load_win32com(finder, module):
 
 
 def load_win32file(finder, module):
-    """the win32api module implicitly loads the pywintypes module; make sure
-       this happens."""
+    """the win32file module implicitly loads the pywintypes and win32timezone
+       module; make sure this happens."""
     finder.IncludeModule("pywintypes")
+    finder.IncludeModule("win32timezone")
 
 
 def load_wx_lib_pubsub_core(finder, module):
@@ -737,13 +812,6 @@ def load_zope_component(finder, module):
     finder.IncludeModule("pkg_resources")
 
 
-def missing_cElementTree(finder, caller):
-    """the cElementTree has been incorporated into the standard library in
-       Python 2.5 so ignore its absence if it cannot found."""
-    if sys.version_info[:2] >= (2, 5):
-        caller.IgnoreName("cElementTree")
-
-
 def missing_EasyDialogs(finder, caller):
     """the EasyDialogs module is not normally present on Windows but it also
        may be so instead of excluding it completely, ignore it if it can't be
@@ -772,11 +840,36 @@ def missing_readline(finder, caller):
         caller.IgnoreName("readline")
 
 
-def missing_xml_etree(finder, caller):
-    """the xml.etree package is new for Python 2.5 but it is common practice
-       to use a try..except.. block in order to support versions earlier than
-       Python 2.5 transparently; ignore the absence of the package in this
-       situation."""
-    if sys.version_info[:2] < (2, 5):
-        caller.IgnoreName("xml.etree")
+def load_zmq(finder, module):
+    """the zmq package loads zmq.backend.cython dynamically and links
+    dynamically to zmq.libzmq."""
+    finder.IncludePackage("zmq.backend.cython")
+    if sys.platform == "win32":
+        # Not sure yet if this is cross platform
+        # Include the bundled libzmq library, if it exists
+        try:
+            import zmq.libzmq
+            srcFileName = os.path.basename(zmq.libzmq.__file__)
+            finder.IncludeFiles(
+                os.path.join(module.path[0], srcFileName), srcFileName)
+        except ImportError:
+            pass  # No bundled libzmq library
+
+
+def load_clr(finder, module):
+    """the pythonnet package (imported as 'clr') needs Python.Runtime.dll
+    in runtime"""
+    module_dir = os.path.dirname(module.file)
+    dll_name = 'Python.Runtime.dll'
+    finder.IncludeFiles(os.path.join(module_dir, dll_name),
+            os.path.join("lib", dll_name))
+
+
+def load_sqlite3(finder, module):
+    """In Windows, the sqlite3 module requires an additional dll sqlite3.dll to
+       be present in the build directory."""
+    if sys.platform == "win32":
+        dll_name = "sqlite3.dll"
+        dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
+        finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
 
